#include <algorithm>
#include <cmath>
#include <iostream>
#include <sstream>
#include <string>

#include "form.h"
using namespace std;


Form::Form(double x, double y)  
    :x(x), y(y) { }

double Form::get_x() 
{
    return x;
}

double Form::get_y() 
{
    return y;
}


Circle::Circle(double x, double y, double radius): 
    Form(x, y), 
    radius(radius) { }

void Circle::affiche()  
{
    cout << "CIRCLE with center:(" << Form::get_x() << ", " << Form::get_y() 
         << " ) and radius " << radius << endl;
} 

double Circle::get_radius()
{
    return radius;
}

Bbox Circle::get_bbox() 
{
    return Bbox(x, y, radius*2, radius*2);
}


Square::Square(double x, double y, double side): 
    Form(x, y), 
    side(side) { }

void Square::affiche()  
{
    cout << "SQUARE with center:(" << Form::get_x() << ", " << Form::get_y() 
         << " ) and side " << side << endl;
} 

double Square::get_side()
{
    return side;
}

Bbox Square::get_bbox() 
{
    return Bbox(x, y, side, side);
}


Rectangle::Rectangle(double x, double y, double width, double height): 
    Form(x, y), 
    width(width), height(height) { }

void Rectangle::affiche()  
{
    cout << "RECTANGLE with center:(" << Form::get_x() << ", " << Form::get_y() 
         << " ), width: " << width << " and height: " << height << endl;
} 

double Rectangle::get_width()
{
    return width;
}

double Rectangle::get_height()
{
    return height;
}

Bbox Rectangle::get_bbox() 
{
    return Bbox(x, y, width, height);
}

Bbox Rectangle::merge_bbox(Bbox& other)
{
    double x_min, y_min, x_max, y_max;

    x_min = min(x - width/2,  other.x - other.width/2);
    y_min = min(y - height/2, other.y - other.height/2);
    x_max = max(x + width/2,  other.x + other.width/2);
    y_max = max(y + height/2, other.y + other.height/2);

    return Bbox((x_min+x_max)/2, (y_min+y_max)/2, 
                (x_max-x_min),
                (y_max-y_min));
}

bool Rectangle::intersect_bbox(Bbox& other)
{
	double x_distance, y_distance, x_border, y_border;

    x_distance  = fabs(x - other.x);
    y_distance  = fabs(y - other.y);

    x_border = (width  + other.width)/2;
    y_border = (height + other.height)/2;

    if((x_distance < x_border) && (y_distance < y_border))
        return true;
    else
        return false;
}


ostream & operator<<(ostream & o, Form * f)
{
	f->affiche();
    return o;
}

