#include <iostream>
#include <cairomm/context.h>
#include "myarea.h"

// default Model Framing and window parameters
static Frame default_frame = {-150., 150., -100., 100., 1.5, 300, 200}; 


MyArea::MyArea()
{
    set_content_width(300);
	set_content_height(200);
	
	set_draw_func(sigc::mem_fun(*this, &MyArea::on_draw));

	setFrame(default_frame);
}

MyArea::~MyArea()
{
}

void MyArea::setFrame(Frame f)
{
	if((f.xMin <= f.xMax) and (f.yMin <= f.yMax) and (f.height > 0))
	{
		f.asp = f.width/f.height;
		frame = f;
	}
	else
		std::cout << "incorrect Model framing or window parameters" << std::endl;
} 

static void orthographic_projection(const Cairo::RefPtr<Cairo::Context>& cr, 
									Frame frame)
{
	// déplace l'origine au centre de la fenêtre
	cr->translate(frame.width/2, frame.height/2);
  
	// normalise la largeur et hauteur aux valeurs fournies par le cadrage
	// ET inverse la direction de l'axe Y
	cr->scale(frame.width/(frame.xMax - frame.xMin), 
             -frame.height/(frame.yMax - frame.yMin));
  
	// décalage au centre du cadrage
	cr->translate(-(frame.xMin + frame.xMax)/2, -(frame.yMin + frame.yMax)/2);
}

void MyArea::on_draw(const Cairo::RefPtr<Cairo::Context>& cr,
                     int width, int height)
{ 
	frame.width = width ;
	frame.height= height;
  
	// le dessin reste dans la fenêtre si on change sa taille 
	// mais une distorsion est introduite si le nouveau aspect ratio
	// de la fenêtre est différente de celui du cadrage (dans frame).
	orthographic_projection(cr, frame); 

	//set width and color
	cr->set_line_width(10);
	cr->set_source_rgb(0.8, 0., 0.0);

	//Now we can draw directly in the Model space
	
	// first line
	cr->move_to(-50., -50);
	cr->line_to(50. , 50);
	//draw second line
	cr->move_to(-50. , 50);
	cr->line_to(50.  ,-50.);

	cr->stroke();
}

